@@ -807,7 +807,7 @@ sub gsub {
       push @$x2_elements, $x_e;
     }
     else {
-      my $x = $x_e->{cv};
+      my $x = $x_e->cv;
       if ($ignore_case) {
         $x =~ s/$pattern/$replacement/gi;
       }
@@ -835,7 +835,7 @@ sub grep {
     my $x_e = $x1_x_elements->[$i];
     
     unless ($x_e->is_na) {
-      my $x = $x_e->{cv};
+      my $x = $x_e->cv;
       if ($ignore_case) {
         if ($x =~ /$pattern/i) {
           push @$x2_elements, Rstats::ElementFunc::double($i + 1);
@@ -852,6 +852,142 @@ sub grep {
   return c($x2_elements);
 }
 
+sub c {
+  my @elements_tmp1 = @_;
+  
+  # Fix elements
+  my $elements_tmp2;
+  if (@elements_tmp1 == 0) {
+    return NULL();
+  }
+  elsif (@elements_tmp1 > 1) {
+    $elements_tmp2 = \@elements_tmp1;
+  }
+  else {
+    $elements_tmp2 = $elements_tmp1[0];
+  }
+
+  my $elements = [];
+  if (defined $elements_tmp2) {
+    if (ref $elements_tmp2 eq 'ARRAY') {
+      for my $element (@$elements_tmp2) {
+        if (ref $element eq 'ARRAY') {
+          push @$elements, @$element;
+        }
+        elsif (ref $element eq 'Rstats::Container::Array') {
+          push @$elements, @{$element->elements};
+        }
+        else {
+          push @$elements, $element;
+        }
+      }
+    }
+    elsif (ref $elements_tmp2 eq 'Rstats::Container::Array') {
+      $elements = $elements_tmp2->elements;
+    }
+    else {
+      $elements = [$elements_tmp2];
+    }
+  }
+  else {
+    return NA();
+  }
+  
+  # Check elements
+  my $mode_h = {};
+  for my $element (@$elements) {
+    
+    if (!ref $element) {
+      if (Rstats::Util::is_perl_number($element)) {
+        $element = Rstats::ElementFunc::double($element);
+        $mode_h->{double}++;
+      }
+      else {
+        $element = Rstats::ElementFunc::character("$element");
+        $mode_h->{character}++;
+      }
+    }
+    elsif ($element->is_na) {
+      next;
+    }
+    elsif ($element->is_character) {
+      $mode_h->{character}++;
+    }
+    elsif ($element->is_complex) {
+      $mode_h->{complex}++;
+    }
+    elsif ($element->is_double) {
+      $mode_h->{double}++;
+    }
+    elsif ($element->is_integer) {
+      $element = Rstats::ElementFunc::double($element->value);
+      $mode_h->{double}++;
+    }
+    elsif ($element->is_logical) {
+      $mode_h->{logical}++;
+    }
+  }
+
+  # Array
+  my $x1 = NULL();
+  $x1->elements($elements);
+
+  # Upgrade elements and type
+  my @modes = keys %$mode_h;
+  if (@modes > 1) {
+    if ($mode_h->{character}) {
+      $x1 = $x1->as_character;
+    }
+    elsif ($mode_h->{complex}) {
+      $x1 = $x1->as_complex;
+    }
+    elsif ($mode_h->{double}) {
+      $x1 = $x1->as_double;
+    }
+  }
+  else {
+    $x1->mode($modes[0] || 'logical');
+  }
+  
+  return $x1;
+}
+
+sub C {
+  my $seq_str = shift;
+
+  my $by;
+  my $mode;
+  if ($seq_str =~ s/^(.+)\*//) {
+    $by = $1;
+  }
+  
+  my $from;
+  my $to;
+  if ($seq_str =~ /([^\:]+)(?:\:(.+))?/) {
+    $from = $1;
+    $to = $2;
+    $to = $from unless defined $to;
+  }
+  
+  my $vector = seq({from => $from, to => $to, by => $by});
+  
+  return $vector;
+}
+
+sub col {
+  my $x1 = shift;
+  
+  my $nrow = nrow($x1)->value;
+  my $ncol = ncol($x1)->value;
+  
+  my @values;
+  for my $col (1 .. $ncol) {
+    push @values, ($col) x $nrow;
+  }
+  
+  return array(\@values, [$nrow, $ncol]);
+}
+
 sub chartr {
   my ($x1_old, $x1_new, $x1_x) = args(['old', 'new', 'x'], @_);
   
@@ -864,7 +1000,7 @@ sub chartr {
       push @$x2_elements, $x_e;
     }
     else {
-      my $x = $x_e->{cv};
+      my $x = $x_e->cv;
       $old =~ s#/#\/#;
       $new =~ s#/#\/#;
       eval "\$x =~ tr/$old/$new/";
@@ -912,6 +1048,15 @@ sub charmatch {
   return c($x2_elements);
 }
 
+sub Conj {
+  my $x1 = to_c(shift);
+  
+  my @a2_elements = map { Rstats::ElementFunc::Conj($_) } @{$x1->elements};
+  my $x2 = $x1->clone(elements => \@a2_elements);
+  
+  return $x2;
+}
+
 sub Re {
   my $x1 = to_c(shift);
   
@@ -932,15 +1077,21 @@ sub Im {
   return $x2;
 }
 
-sub Conj {
-  my $x1 = to_c(shift);
+sub nrow {
+  my $x1 = shift;
   
-  my @a2_elements = map { Rstats::ElementFunc::Conj($_) } @{$x1->elements};
-  my $x2 = $x1->clone(elements => \@a2_elements);
-  
-  return $x2;
+  if ($x1->is_data_frame) {
+    return c($x1->{row_length});
+  }
+  elsif ($x1->is_list) {
+    return Rstats::Func::NULL();
+  }
+  else {
+    return c($x1->dim->values->[0]);
+  }
 }
 
+
 sub negation {
   my $x1 = shift;
   
@@ -2492,34 +2643,6 @@ sub sum {
   return c($sum);
 }
 
-sub col {
-  my $x1 = shift;
-  
-  my $nrow = nrow($x1)->value;
-  my $ncol = ncol($x1)->value;
-  
-  my @values;
-  for my $col (1 .. $ncol) {
-    push @values, ($col) x $nrow;
-  }
-  
-  return array(\@values, [$nrow, $ncol]);
-}
-
-sub nrow {
-  my $x1 = shift;
-  
-  if ($x1->is_data_frame) {
-    return c($x1->{row_length});
-  }
-  elsif ($x1->is_list) {
-    return Rstats::Func::NULL();
-  }
-  else {
-    return c($x1->dim->values->[0]);
-  }
-}
-
 sub ncol {
   my $x1 = shift;
   
@@ -2614,128 +2737,6 @@ sub seq {
   }
 }
 
-sub c {
-  my @elements_tmp1 = @_;
-  
-  # Fix elements
-  my $elements_tmp2;
-  if (@elements_tmp1 == 0) {
-    return NULL();
-  }
-  elsif (@elements_tmp1 > 1) {
-    $elements_tmp2 = \@elements_tmp1;
-  }
-  else {
-    $elements_tmp2 = $elements_tmp1[0];
-  }
-
-  my $elements = [];
-  if (defined $elements_tmp2) {
-    if (ref $elements_tmp2 eq 'ARRAY') {
-      for my $element (@$elements_tmp2) {
-        if (ref $element eq 'ARRAY') {
-          push @$elements, @$element;
-        }
-        elsif (ref $element eq 'Rstats::Container::Array') {
-          push @$elements, @{$element->elements};
-        }
-        else {
-          push @$elements, $element;
-        }
-      }
-    }
-    elsif (ref $elements_tmp2 eq 'Rstats::Container::Array') {
-      $elements = $elements_tmp2->elements;
-    }
-    else {
-      $elements = [$elements_tmp2];
-    }
-  }
-  else {
-    return NA();
-  }
-  
-  # Check elements
-  my $mode_h = {};
-  for my $element (@$elements) {
-    
-    if (!ref $element) {
-      if (Rstats::Util::is_perl_number($element)) {
-        $element = Rstats::ElementFunc::double($element);
-        $mode_h->{double}++;
-      }
-      else {
-        $element = Rstats::ElementFunc::character("$element");
-        $mode_h->{character}++;
-      }
-    }
-    elsif ($element->is_na) {
-      next;
-    }
-    elsif ($element->is_character) {
-      $mode_h->{character}++;
-    }
-    elsif ($element->is_complex) {
-      $mode_h->{complex}++;
-    }
-    elsif ($element->is_double) {
-      $mode_h->{double}++;
-    }
-    elsif ($element->is_integer) {
-      $element = Rstats::ElementFunc::double($element->value);
-      $mode_h->{double}++;
-    }
-    elsif ($element->is_logical) {
-      $mode_h->{logical}++;
-    }
-  }
-
-  # Array
-  my $x1 = NULL();
-  $x1->elements($elements);
-
-  # Upgrade elements and type
-  my @modes = keys %$mode_h;
-  if (@modes > 1) {
-    if ($mode_h->{character}) {
-      $x1 = $x1->as_character;
-    }
-    elsif ($mode_h->{complex}) {
-      $x1 = $x1->as_complex;
-    }
-    elsif ($mode_h->{double}) {
-      $x1 = $x1->as_double;
-    }
-  }
-  else {
-    $x1->mode($modes[0] || 'logical');
-  }
-  
-  return $x1;
-}
-
-sub C {
-  my $seq_str = shift;
-
-  my $by;
-  my $mode;
-  if ($seq_str =~ s/^(.+)\*//) {
-    $by = $1;
-  }
-  
-  my $from;
-  my $to;
-  if ($seq_str =~ /([^\:]+)(?:\:(.+))?/) {
-    $from = $1;
-    $to = $2;
-    $to = $from unless defined $to;
-  }
-  
-  my $vector = seq({from => $from, to => $to, by => $by});
-  
-  return $vector;
-}
-
 sub numeric {
   my $num = shift;
   
